import * as React from 'react';
import { ObjectOmit } from '../utils';
import { Button } from './Button';
/**
 * A generic button wrapper; it handles processing/success/failure states
 *
 * If buttonState is passed in as a prop, all internal states are overridden.
 *
 * stableSuccess should not be changed for the lifetime of the component
 *
 * Otherwise, the following happens:
 *
 *   internalState = null
 *     |
 *     ? click
 *     | --> invoke onClick and grab promise
 *     V
 *   internalState = processing
 *     |                    \
 *     ? promise success     ------? promise failure
 *     |                            \
 *     V                             V
 *   internalState = success       internalState = error
 *     |                 \                           \
 *     |                  --------                    --------------------
 *     |                          \                                       \
 *     ? stableSuccess is true     ? stableSuccess is false                |
 *     |                           |                                       |
 *     O wait for prop change      * < wait for timerMillis milliseconds > *
 *                                 |                                      /
 *                                 |              ------------------------
 *                                 |             /
 *                                 V             V
 *                               internalState = null
 *
 */
/**
 * ready or not-allowed; use it if you want button to handle its internal state and onClick is a promise
 */
export interface StatefulButtonRequiredProps extends Button.Props {
    /** callback */
    onClick: (e: React.SyntheticEvent<HTMLDivElement>) => Promise<any>;
    /** ready, not-allowed, success, use it if you want button to be a functional component */
    baseState?: StatefulButton.ButtonBaseState;
}
export interface StatefulButtonDefaultProps {
    /** keep success state  */
    stableSuccess: boolean;
    /** time in millisecons to wait before state reset  */
    timerMillis: number;
}
export declare namespace StatefulButton {
    type ButtonBaseState = 'ready' | 'success' | 'not-allowed';
    type Props = ObjectOmit<Button.Props, 'onClick'> & StatefulButtonRequiredProps & Partial<StatefulButtonDefaultProps>;
}
export type State = {
    internalState: 'error' | 'processing' | 'success' | null;
};
export declare class StatefulButton extends React.PureComponent<StatefulButton.Props, State> {
    private timeoutId;
    private resetInternalStateAfterProcessing;
    private _isMounted;
    static defaultProps: StatefulButtonDefaultProps;
    state: {
        internalState: null;
    };
    constructor(props: StatefulButton.Props);
    componentDidMount(): void;
    componentWillUnmount(): void;
    doResetInternalState: () => void;
    componentDidUpdate(prevProps: StatefulButton.Props): void;
    doResetInternalStateAfterTimer: () => void;
    attachPromiseHandlers: (promise: Promise<void>) => void;
    getButtonState: () => "error" | "not-allowed" | "success" | "ready" | "processing" | undefined;
    onClick: (e: React.SyntheticEvent<HTMLDivElement>) => void;
    render(): JSX.Element;
}
